<?php
namespace Folha\Logic\Cadastros\Vinculo;


use \Folha\Logic\Validacoes\Vinculo\DepVinculo as ValidacaoDepVinculo;

class CadastroAutomaticoDependentes
{

	const TIPO_RELACIONAMENTO_FILHO       = 3;
	const TIPO_RELACIONAMENTO_TUTELADO    = 5;
	const TIPO_RELACIONAMENTO_PENSIONISTA = 6;

	const TIPODEPENDE_SALARIO_FAMILIA     = 1;
	const TIPODEPENDE_IMPOSTO_DE_RENDA    = 2;
	const TIPODEPENDE_PENSAO_JUDICIAL     = 3;


	/**
	 * @var $vinculo GpevinculomEntity
	 */
	private $vinculo;

	private $registrosSalvos;


	public function __construct($nrvinculom, $nrorg) 
	{
		$this->vinculo = \Gpevinculom::getObjectByWhere(
			"NRVINCULOM = ? AND NRORG = ?",
			array($nrvinculom, $nrorg)
		);

		if (!$this->vinculo instanceof \GpevinculomEntity) {
			throw new \Exception ("vinculo nao encontrado.");
		}
	}

	/**
     * Cadastra todos os registros validos para fpa_depvinculo.
     *
     * @param $dependente array Lista contendo os dados dos dependentes. Este array deve possuir os indices de 
     *                          NRPARCDEPENDE, NRORG e NRTIPORELACIONA para cada elemento.
     * 
     * @return String
     *
     * @author paulopereira - 20/11/2013
     */
	public function executaCadastro($dependentes = array()) 
	{
		try {
			foreach ($dependentes as $dependente) {
				$this->criaDependentes($dependente["NRPARCDEPENDE"], $dependente["NRORG"], $dependente["NRTIPORELACIONA"]);
			}
		} catch (\Exception $e) {
			throw $e;
		}
	}

	public function getNrRegistrosSalvos() 
	{
		return $this->registrosSalvos;
	}

	/**
     * Cria os registros de depvinculo necessarios para cada caso. Ex.: para relacionamentos do tipo "FILHO", que tenham
     * idade menor que 14 anos, este metodo criara registros de depvinculo para salario familia e imposto de renda.
     *
     * @param $nrparcdepende   string Chave do parceiro de negocio do dependente do vinculo.
     * @param $nrorg           string Chave da organizacao do vinculo.
     * @param $nrtiporelaciona string tipo de relacionamento existente entre vinculo e dependente.
     * 
     * @return String
     *
     * @author paulopereira - 20/11/2013
     */
	private function criaDependentes($nrparcdepende, $nrorg, $nrtiporelaciona) 
	{
		try {
			$pessoaDependente = $this->getPessoaDependente($nrorg, $nrparcdepende);
			$this->criaDepvinculos($pessoaDependente, $nrtiporelaciona);
		} catch (\Exception $e) {
			throw $e;
		}

	}

	/**
     * Retorna uma instancia do tipo \GpepessoaEntity para a pessoa no sistema, referente ao parceiro de negocio 
     * identificado pelos parametros. Caso o registro de gpe_pessoa nao exista, este devera ser criado, e posteriormente
     * retornado.
     *
     * @param $nrorg           string Chave da organizacao do vinculo.
     * @param $nrparcdepende   string Chave do parceiro de negocio do dependente do vinculo.
     * 
     * @return String
     *
     * @author paulopereira - 22/11/2013
     */
	private function getPessoaDependente($nrorg, $nrparcdepende) 
	{
		$pessoaDependente = \GpepessoaEntity::retornaPessoaPorParceiro($nrorg, $nrparcdepende);
		if (!$pessoaDependente instanceof \GpepessoaEntity) {
			$gpepessoaEntity = new \GpepessoaEntity();
			$gpepessoaEntity->incluiPessoa($nrparcdepende, $nrorg);
		}

		return $pessoaDependente;
	}

	/**
	 * @todo Verificar se todos os tipos de relacionamentos foram contemplados neste metodo.
     * Cria todas as instancias de fpa_depvinculo necessarias a determinado relacionamento do vinculo. O tipo de 
     * relacionamentos entre eles definira quais tipos de dependencia serao criados.
     *
     * @param $pessoa          \GpepessoaEntity Instancia de pessoa representando o individuo dependente.
     * @param $nrtiporelaciona string           Tipo de relacionamento entre vinculo e parceiro relacionado.
     * 
     * @return String
     *
     * @author paulopereira - 22/11/2013
     */
	private function criaDepvinculos(\GpepessoaEntity $pessoa, $nrtiporelaciona) 
	{
		$nrorg       = $this->vinculo->getNrorg();
		$nrvinculom  = $this->vinculo->getNrvinculom();
		$nrpessoaDep = $pessoa->getNrpessoa();

		try {
			switch ($nrtiporelaciona) {
				case self::TIPO_RELACIONAMENTO_FILHO:
					$this->criaDepvinculosFilho($nrorg, $nrvinculom, $nrpessoaDep);
					break;
				
				case self::TIPO_RELACIONAMENTO_PENSIONISTA:
					$this->criaDepvinculosPensionista($nrorg, $nrvinculom, $nrpessoaDep);
					break;

				case self::TIPO_RELACIONAMENTO_TUTELADO:
					$this->criaDepvinculosTutelado($nrorg, $nrvinculom, $nrpessoaDep);
					break;

				default:
					break;
			}
		} catch (\Exception $e) {
			throw $e;
		}

	}

	/**
     * Cria os tipos de dependencias existentes para os relacionamentos do tipo "Filho (a)".
     *
     * @param $nrorg       integer Identificador da organizacao do vinculo.
     * @param $nrvinculom  integer Identificador do vinculo.
     * @param $nrpessoaDep integer Identificador da pessoa dependente.
     * 
     * @return String
     *
     * @author paulopereira - 22/11/2013
     */
	private function criaDepvinculosFilho($nrorg, $nrvinculom, $nrpessoaDep) 
	{
		try {
			if (ValidacaoDepVinculo::validaSalarioFamilia($nrpessoaDep, $nrorg)) {
				$this->insereDepVinculo($nrorg, $nrpessoaDep, self::TIPODEPENDE_SALARIO_FAMILIA, $nrvinculom);
			}

			$this->insereDepVinculo($nrorg, $nrpessoaDep, self::TIPODEPENDE_IMPOSTO_DE_RENDA, $nrvinculom);
		} catch (\Exception $e) {
			throw $e;
		}
	}

	/**
	 * @todo revisar quais tipos de dependencias deverao ser criadas para este relacionamento.
     * Cria os tipos de dependencias existentes para os relacionamentos do tipo "Pensionista".
     *
     * @param $nrorg       integer Identificador da organizacao do vinculo.
     * @param $nrvinculom  integer Identificador do vinculo.
     * @param $nrpessoaDep integer Identificador da pessoa dependente.
     * 
     * @return String
     *
     * @author paulopereira - 22/11/2013
     */
	private function criaDepvinculosPensionista($nrorg, $nrvinculom, $nrpessoaDep) 
	{
		try {
			$this->insereDepVinculo($nrorg, $nrpessoaDep, self::TIPODEPENDE_PENSAO_JUDICIAL, $nrvinculom);
		} catch (\Exception $e) {
			throw $e;
		}
	}

	/**
	 * @todo revisar quais tipos de dependencias deverao ser criadas para este relacionamento.
     * Cria os tipos de dependencias existentes para os relacionamentos do tipo "Tutelado".
     *
     * @param $nrorg       integer Identificador da organizacao do vinculo.
     * @param $nrvinculom  integer Identificador do vinculo.
     * @param $nrpessoaDep integer Identificador da pessoa dependente.
     * 
     * @return String
     *
     * @author paulopereira - 22/11/2013
     */
	private function criaDepvinculosTutelado($nrorg, $nrvinculom, $nrpessoaDep) 
	{
		try {
			$this->insereDepVinculo($nrorg, $nrpessoaDep, self::TIPODEPENDE_IMPOSTO_DE_RENDA, $nrvinculom);
		} catch (\Exception $e) {
			throw $e;
		}
	}

	/**
     * Insere um novo registro na tabela fpa_depvinculo. Este metodo somente criara um novo se nao houver nenhum outro 
     * fazendo a mesma dependencia.
     *
     * @param $nrorg         integer Identificador da organizacao do vinculo.
     * @param $nrpessoaDep   integer Identificador da pessoa dependente.
     * @param $nrtipodepende integer Tipo de dependencia.
     * @param $nrvinculom    integer Identificador do vinculo.
     * 
     * @return String
     *
     * @author paulopereira - 22/11/2013
     */
	private function insereDepVinculo($nrorg, $nrpessoaDep, $nrtipodepende, $nrvinculom) 
	{
		$depvinculo = new \FpadepvinculoEntity();
		if (\Fpadepvinculo::existeDependencia($nrorg, $nrpessoaDep, $nrtipodepende, $nrvinculom) === false) {
			try {
				$depvinculo->insereDependenteVinculo($nrorg, $nrpessoaDep, $nrtipodepende, $nrvinculom);
				$depvinculo->persist();
				$this->registrosSalvos++;
			} catch (\Exception $e) {
				throw $e;
			}
		}
	}

}